RollMaster = LibStub("AceAddon-3.0"):NewAddon("RollMaster", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

local df = LibStub("LibDeformat-3.0")
local rm = RollMaster
local defaults = {
  profile = {
    timeout = 30,
    keptrolls = 5,
  }
}
local options = {
    name = "RollMaster",
    handler = RollMaster,
    type = "group",
  	get = function(info) return rm.db[info[#info]] end,
  	set = function(info, value) rm.db[info[#info]] = value end,
    args = {
      timeout = {
        name = "Timeout",
        desc = "The time from a loot roll starting to it timing out",
        type = "range",
        min = 10,
        max = 60,
        step = 1,
      },
      keptrolls = {
        name = "Kept Rolls",
        desc = "The number of historical rolls to keep",
        type = "range",
        min = 0,
        max = 20,
        step = 1,
      },
      list = {
        name = "List rolls",
        desc = "List stored rolls",
        type = "execute",
        func = "ListRolls",
        hidden = true,
      }
    },
}


function RollMaster:OnInitialize()
  -- Code that you want to run when the addon is first loaded goes here.
  self.db_root = LibStub("AceDB-3.0"):New("RollMasterDB",defaults)
  self.db=self.db_root.profile
	local acreg = LibStub("AceConfigRegistry-3.0")
	acreg:RegisterOptionsTable("RollMaster", options)
	acreg:RegisterOptionsTable("RollMaster Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db_root))
	local acdia = LibStub("AceConfigDialog-3.0")
	acdia:AddToBlizOptions("RollMaster", "RollMaster")
	acdia:AddToBlizOptions("RollMaster Profiles", "Profiles", "RollMaster")
  self:RegisterChatCommand("rollmaster", "ChatCommand")
  self:RegisterChatCommand("rm", "ChatCommand")
end

function RollMaster:OnEnable()
  self:Print("Hello from the Roll Master")
  self:RegisterEvent("CHAT_MSG_SYSTEM")
end

function RollMaster:OnDisable()
  self:Print("Goodbye from the Roll Master")
end

function RollMaster:ChatCommand(input)
  if not input or input:trim() == "" then
    InterfaceOptionsFrame_OpenToCategory("RollMaster")
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(RollMaster, "rm", "RollMaster", input)
  end
end

function RollMaster:CHAT_MSG_SYSTEM(event, msg)
  local player,rolled,low,high = df:Deformat(msg, RANDOM_ROLL_RESULT)
  if player then
    self:ProcessRoll(player,rolled,low,high)
  end
end

local function getRollCount()
  local rmcnt=GetNumRaidMembers()
  local ptcnt=GetNumPartyMembers()
  if rmcnt>0 then 
    return rmcnt
  else
    return ptcnt+1
  end
end

function RollMaster:ProcessRoll(pl,rl,lo,hi)
  local cr=self.currentRoll or {}
  table.insert(cr,{pl,rl,lo,hi})
  self.currentRoll=cr
  if #cr==1 then
    self.rollCount=getRollCount()
    self.timestamp=date()
    self.rollTimer=self:ScheduleTimer("RollComplete",self.db.timeout,"Time Up")
    self:Print(format("Starting a new roll for %d people. %d seconds remaining",self.rollCount,self.db.timeout))
  end
  if #cr==self.rollCount then
    self:RollComplete("All rolled")
  end
end

function RollMaster:RollComplete(reason)
  table.sort(self.currentRoll, function(a,b) return a[2]>b[2] end)
  self:Print(format("Roll completed: %s",reason))
  for idx,roll in ipairs(self.currentRoll) do
    self:Print(format("%d. %s rolled a %d (%d-%d)",idx,roll[1],roll[2],roll[3],roll[4]))
  end
  self.currentRoll.reason=reason
  self.currentRoll.timestamp=self.timestamp
  self.db.oldrolls=self.db.oldrolls or {}
  table.insert(self.db.oldrolls,self.currentRoll)
  self.currentRoll=nil
  self:CancelTimer(self.rollTimer,true)
  self.rollTimer=nil
  if #self.db.oldrolls>self.db.keptrolls then
    table.remove(self.db.oldrolls,1)
  end
end

function RollMaster:ListRolls()
  if not self.db.oldrolls then
    self:Print("No rolls stored")
  else
    for idx,roll in ipairs(self.db.oldrolls) do
      ts=roll.timestamp
      re=roll.reason
      self:Print(format("Roll at %s: %s",ts,re))
      for idx,pl in ipairs(roll) do
        self:Print(format("%d. %s rolled a %d",idx,pl[1],pl[2]))
      end
    end
  end
end  
    
